@@ -80,6 +80,7 @@ class CompositeDialogueFlow:
         :param debugging:
         :return: None
         """
+        self.controller().vars()['__user_utterance__'] = natural_language
         try:
             self.controller().apply_update_rules(natural_language, debugging=debugging)
             next_state = self.controller().state()
@@ -119,9 +120,15 @@ class CompositeDialogueFlow:
         self.component(namespace).set_state(old_state)
         self.set_controller(namespace)
         self.controller().set_speaker(speaker)
-        if speaker == DialogueFlow.Speaker.USER and not self.controller().state_settings(state).user_multi_hop:
+        if isinstance(state, tuple):
+            umh = self.state_settings(*state).user_multi_hop
+            smh = self.state_settings(*state).system_multi_hop
+        else:
+            umh = self.state_settings(namespace, state).user_multi_hop
+            smh = self.state_settings(namespace, state).system_multi_hop
+        if speaker == DialogueFlow.Speaker.USER and not umh:
             self.controller().change_speaker()
-        elif speaker == DialogueFlow.Speaker.SYSTEM and not self.controller().state_settings(state).system_multi_hop:
+        elif speaker == DialogueFlow.Speaker.SYSTEM and not smh:
             self.controller().change_speaker()
         self.controller().set_state(state)
 
@@ -189,13 +196,16 @@ class CompositeDialogueFlow:
     def component(self, namespace):
         return self._components[namespace]
 
+    def components(self):
+        return self._components.values()
+
     def set_state(self, state):
         state = module_state(state)
         if isinstance(state, tuple):
             if self.component(state[0]) != self.controller():
                 self.component(state[0]).set_state(self.controller().state())  # so __system_state__ is set properly
                 self.set_controller(state[0])
-                state = state[1]
+            state = state[1]
         self.controller().set_state(state)
 
     def set_controller(self, controller_name):
