@@ -25,14 +25,17 @@ from pathos.multiprocessing import ProcessingPool as Pool
 
 def module_source_target(source, target):
     if isinstance(source, str) and ':' in source:
-        source = tuple(source.split(':'))
+        i = source.find(':')
+        source = (source[:i], source[i+1:])
     if isinstance(target, str) and ':' in target:
-        target = tuple(target.split(':'))
+        i = target.find(':')
+        target = (target[:i], target[i + 1:])
     return source, target
 
 def module_state(state):
     if isinstance(state, str) and ':' in state:
-        state = tuple(state.split(':'))
+        i = state.find(':')
+        state = (state[:i], state[i + 1:])
     return state
 
 def precache(transition_datas):
@@ -142,18 +145,22 @@ class DialogueFlow:
             'UNX': Unexpected(),
             'INT': Intent(),
             'GEXT': goal_exit_macro,
-            'GOAL': GoalPursuit(goal_exit_macro),
+            'GOAL': GoalPursuit(goal_exit_macro, self),
             'GCOM': GoalCompletion(self),
             'GRET': GoalReturn(self),
             'DEFAULT': Default(),
             'VT': VirtualTransitions(self),
             'GSRET': SetGoalReturnPoint(),
-            'TARGET': Target()
+            'TARGET': Target(),
+            'CE': CanEnter(self),
+            'RW': Rewrite(),
+            'CONTRACTIONS': ExpandContractions()
         }
         if macros:
             self._macros.update(macros)
         self._rules = UpdateRules(vars=self._vars, macros=self._macros)
         self.add_state(end_state)
+        self._vars['__user_utterance__'] = None
 
 
     # TOP LEVEL: SYSTEM-LEVEL USE CASES
@@ -206,6 +213,7 @@ class DialogueFlow:
         :return: None
         """
         t1 = time()
+        natural_language = ''.join([c.lower() for c in natural_language if c.isalpha() or c == ' '])
         self._transitions.clear()
         self.apply_update_rules(natural_language, debugging)
         visited = {self.state()}
@@ -343,6 +351,8 @@ class DialogueFlow:
         :param debugging:
         :return: a <state, response> tuple representing the successor state and response
         """
+        if '__gate__' in self._vars:
+            del self._vars['__gate__']
         state = module_state(state)
         ti = time()
         if state is None:
@@ -361,6 +371,7 @@ class DialogueFlow:
             transition_items.append((natex, transition, score))
         for natex, transition, score in transition_items:
             t1 = time()
+            transition_transition_enter = None
             vars = HashableDict(self._vars)
             self._potential_transition = transition # MOVED, todo
             try:
@@ -379,10 +390,12 @@ class DialogueFlow:
                 target = State(module_state(vars['__target__']))
                 del vars['__target__']
             transition = source, target, speaker
-            if not self.is_module() and isinstance(target, tuple):
-                continue
+            # if not self.is_module() and isinstance(target, tuple):
+            #     continue
             if '->' in transition[1]:
-                transition = (target.split('->')[0], target.split('->')[1], speaker)
+                _src, _tar = target.split('->')[0], target.split('->')[1]
+                _tar = State(module_state(_tar))
+                transition = (_src, _tar, speaker)
                 try:
                     appended_generation = self.transition_natex(*transition).generate(vars=vars, macros=self._macros, debugging=debugging)
                     if appended_generation is None:
@@ -397,7 +410,10 @@ class DialogueFlow:
                     generation = None
             elif isinstance(transition[1], tuple) and '->' in transition[1][1]:
                 namespace = transition[1][0]
-                transition = ((namespace, target[1].split('->')[0]), (namespace, target[1].split('->')[1]), speaker)
+                source, target = (namespace, target[1].split('->')[0]), target[1].split('->')[1]
+                target = State(module_state(target))
+                transition_transition_enter = source
+                transition = (source, target, speaker)
                 try:
                     appended_generation = self.composite_dialogue_flow().transition_natex(
                         namespace, *transition).generate(vars=vars, macros=self._macros, debugging=debugging)
@@ -416,20 +432,36 @@ class DialogueFlow:
                 target = State(module_state(vars['__target__']))
                 del vars['__target__']
             transition = source, target, speaker
-            if self.is_module() and isinstance(target, tuple):
-                enter_natex = self.composite_dialogue_flow().state_settings(*target).enter
-            else:
-                enter_natex = self.state_settings(target).enter
             enter_natex_pass = True
-            if enter_natex is not None:
-                try:
-                    enter_natex_pass = enter_natex.generate(vars=vars, macros=self._macros, debugging=debugging)
-                except Exception as e:
-                    print()
-                    print(e)
-                    print('Enter Natex {}: {} failed'.format(str(target), enter_natex))
-                    print()
-                    enter_natex_pass = None
+            transition_transition_enter_vars = dict(vars)
+            if transition_transition_enter is not None:
+                if self.is_module() and isinstance(transition_transition_enter, tuple):
+                    enter_natex = self.composite_dialogue_flow().state_settings(*transition_transition_enter).enter
+                else:
+                    enter_natex = self.state_settings(transition_transition_enter).enter
+                if enter_natex is not None:
+                    try:
+                        enter_natex_pass = enter_natex.generate(vars=transition_transition_enter_vars, macros=self._macros, debugging=debugging)
+                    except Exception as e:
+                        print()
+                        print(e)
+                        print('Enter Natex {}: {} failed'.format(str(transition_transition_enter), enter_natex))
+                        print()
+                        enter_natex_pass = None
+            if enter_natex_pass:
+                if self.is_module() and isinstance(target, tuple):
+                    enter_natex = self.composite_dialogue_flow().state_settings(*target).enter
+                else:
+                    enter_natex = self.state_settings(target).enter
+                if enter_natex is not None:
+                    try:
+                        enter_natex_pass = enter_natex.generate(vars=vars, macros=self._macros, debugging=debugging)
+                    except Exception as e:
+                        print()
+                        print(e)
+                        print('Enter Natex {}: {} failed'.format(str(target), enter_natex))
+                        print()
+                        enter_natex_pass = None
             if generation is not None and enter_natex_pass is not None:
                 if '__score__' in vars:
                     score = vars['__score__']
@@ -444,8 +476,20 @@ class DialogueFlow:
                         if gate_var_config == vc:
                             gate_closed = True
                     del vars['__gate__']
+                tt_gate_var_config = None
+                tt_gate_target_id = None
+                if transition_transition_enter is not None and '__gate__' in transition_transition_enter_vars:
+                    tt_gate_var_config = transition_transition_enter_vars['__gate__']
+                    tt_gate_target_id = (self.namespace(), transition_transition_enter) if \
+                        (not isinstance(transition_transition_enter, tuple) and self.is_module()) else transition_transition_enter
+                    for vc in self.gates()[tt_gate_target_id]:
+                        if tt_gate_var_config == vc:
+                            gate_closed = True
+                    del transition_transition_enter_vars['__gate__']
+                transition_transition_enter_vars.update(vars)
+                vars = transition_transition_enter_vars
                 if not gate_closed:
-                    transition_options.append((score, generation, transition, vars, gate_var_config, gate_target_id))
+                    transition_options.append((score, generation, transition, vars, gate_var_config, gate_target_id, tt_gate_var_config, tt_gate_target_id))
             t2 = time()
             if debugging:
                 print('Transition {} evaluated in {:.5f}'.format(transition, t2-t1))
@@ -454,9 +498,12 @@ class DialogueFlow:
                 transition_items.append((natex, transition, score))
         self._transitions.clear()
         if transition_options:
-            score, response, transition, vars, gate_var_config, gate_target_id = random_max(transition_options, key=lambda x: x[0])
+            score, response, transition, vars, gate_var_config, gate_target_id, tt_gate_var_config, tt_gate_target_id =\
+                random_max(transition_options, key=lambda x: x[0])
             if gate_var_config is not None:
                 self.gates()[gate_target_id].append(gate_var_config)
+            if tt_gate_var_config is not None:
+                self.gates()[tt_gate_target_id].append(tt_gate_var_config)
             if debugging:
                 updates = {}
                 for k, v in vars.items():
@@ -496,7 +543,12 @@ class DialogueFlow:
         :return: the successor state representing the highest score user transition
                  that matches natural_language, or None if none match
         """
-        natural_language = ''.join([c.lower() for c in natural_language if c.isalpha() or c == ' '])
+        if '__gate__' in self._vars:
+            del self._vars['__gate__']
+        if '__user_utterance__' in self.vars() and self.vars()['__user_utterance__'] is not None:
+            natural_language = self.vars()['__user_utterance__']
+        else:
+            natural_language = ''.join([c.lower() for c in natural_language if c.isalpha() or c == ' '])
         state = module_state(state)
         self._error_transitioned = False
         ti = time()
@@ -618,7 +670,8 @@ class DialogueFlow:
                 data['natex'].precache()
             for rule in self.update_rules().rules:
                 rule.precondition.precache()
-                rule.postcondition.precache()
+                if rule.postcondition:
+                    rule.postcondition.precache()
         else:
             # transition_data_sets = []
             # for i in range(process_num):
@@ -888,6 +941,7 @@ class DialogueFlow:
         self._vars = HashableDict()
         self.vars()['__state__'] = self._initial_state
         self.vars()['__stack__'] = []
+        self.vars()['__user_utterance__'] = None
         self.vars()['__system_state__'] = 'None' if self._initial_speaker == Speaker.USER else self._initial_state
         self.set_state(self._initial_state)
         self._rules.set_vars(self._vars)
@@ -990,6 +1044,10 @@ class DialogueFlow:
             self.add_global_nlu(state, nlu, score)
         self.load_transitions(transitions, Speaker.USER)
 
+    def load_update_rules(self, rules_dict):
+        for pre, post in rules_dict.items():
+            self.update_rules().add(pre, post)
+
     def add_goal(self, id_string, return_state=None, return_phrase=None, doom_counter=None):
         goal = {
             'id': id_string,
