@@ -4,6 +4,8 @@ from lark import Lark, Transformer, Tree, Visitor, Token
 from emora_stdm.state_transition_dialogue_manager.stochastic_options import StochasticOptions
 from emora_stdm.state_transition_dialogue_manager.natex_nlu import NatexNLU
 from copy import deepcopy
+import sys
+import traceback
 
 
 class NatexNLG:
@@ -225,7 +227,9 @@ class NatexNLG:
                     tree.children[0] = macro(self._ngrams, self._vars, macro_args)
                 except Exception as e:
                     print('ERROR: Macro {} raised exception {}'.format(symbol, repr(e)))
+                    traceback.print_exc(file=sys.stdout)
                     tree.children[0] = '_MACRO_EXCEPTION_'
+
                 if self._debugging: print('    {:15} {}'.format(symbol, self._current_compilation(self._tree)))
             else:
                 print('ERROR: Macro {} not found'.format(symbol))
